#include "ModuleBase.h"
#include <algorithm>

namespace mdd {

    size_t ModuleBase::getNumInputs(){
        return inputs.size();
    }

    size_t ModuleBase::getNumOutputs() {
        return outputs.size();
    }


    std::shared_ptr<IInput> ModuleBase::getInput(size_t index) {
        if (index <= inputs.size())
        {
            return inputs[index];
        }
        return nullptr;
    }

    std::shared_ptr<IInput> ModuleBase::getInput(const std::string& input_id){
        for(auto& input : inputs){
            if (input->getID() == input_id) {
                return input;
            }
        }
        return nullptr;
    }

    std::shared_ptr<IOutput> ModuleBase::getOutput(size_t index) {
        if (index <= outputs.size())
        {
            return outputs[index];
        }
        return nullptr;
    }

    std::shared_ptr<IOutput> ModuleBase::getOutput(const std::string& output_id) {
        for (auto& output : outputs) {
            if (output->getID() == output_id) {
                return output;
            }
        }
        return nullptr;
    }

    std::vector<std::shared_ptr<IInput>> ModuleBase::getOptimizableInputs() {
        std::vector<std::shared_ptr<IInput>> ret;
        for (size_t i = 0; i < inputs.size(); i++)
        {
            if (inputs[i]->isOptimizable())
            {
                ret.push_back(inputs[i]);
            }
        }
        
        return ret;
    }

    std::vector<std::shared_ptr<IOutput>> ModuleBase::getOptimizableOutputs() {
        std::vector<std::shared_ptr<IOutput>> ret;
        for (size_t i = 0; i < outputs.size(); i++)
        {
            if (outputs[i]->isOptimizable())
            {
                ret.push_back(outputs[i]);
            }
        }
        
        return ret;
    }

    ModuleBase::ModuleBase(const std::string& base_config)
    {
        json jparse = json::parse(base_config);
        if (jparse.contains("configure"))
        {
            _base_config = jparse;
        }
        else {
            _base_config["configure"] = jparse;
        }
    }
    bool ModuleBase::configure(const std::string& config) {
        json jconfig = json::parse(config);
        bool res = true;
        auto jit = jconfig.find("GUI");
        if (jit != jconfig.end())
        {
            _base_config["GUI"] = jit.value();
        }
        jit = jconfig.find("configure");
        if (jit != jconfig.end()) {
            res = configureChild(jit.value());
        }
        return res;
    }
   
    bool configureThis(const json& config) {
        return true;
    }

    const std::string& ModuleBase::getConfiguration()
    {
        return _base_config.dump();
    }

    std::string ModuleBase::getType() {
        return  type;
    }

    std::string ModuleBase::getGeneratorKey() {
        return  key;
    }

    std::string ModuleBase::setName(const std::string& name) {
        _name = name;
        return getID();
    }

    std::string ModuleBase::getName() {
        return  _name;
    }

    std::string ModuleBase::setAppendix(int appendix) {
        _appendix = appendix;
        return  getID();
    }

    int ModuleBase::getAppendix() {
        return _appendix;
    }

    std::string ModuleBase::getID() {
        return _name + std::to_string(_appendix);
    }

    void ModuleBase::load(const json& j)
    {
        configure(j.dump());

        auto it = j.find("inputs");
        if (it!=j.end())
        {
            const size_t start_counter = inputs.size();
            const json& jin = it.value();
            std::vector<double> default_val = { 1 };
            for (size_t i = start_counter; i < jin.size(); i++)
            {
                inputs.push_back(std::make_shared<Input>(this, "Value", 0, default_val));
                inputs.back()->load(jin[i]);
            }
            for (size_t i = 0; i < jin.size(); i++)
            {
                inputs[i]->setValue()=jin[i]["value"].get<std::vector<double>>();
            }
        }

        it = j.find("outputs");
        if (it != j.end())
        {
            const size_t start_counter = outputs.size();
            const json& jout = it.value();
            std::vector<double> default_val = { 1 };
            for (size_t i = start_counter; i < j["outputs"].size(); i++)
            {
                outputs.push_back(std::make_shared<Output>(this, "Value", 0, default_val));
                outputs.back()->load(jout[i]);
            }
            for (size_t i = 0; i < jout.size(); i++)
            {
                outputs[i]->setValue() = jout[i]["value"].get<std::vector<double>>();
            }
        }
    }
    json ModuleBase::dump()
    {
        _base_config["ID"] = getIdentifier();
        _base_config["type"] = type;
        _base_config["key"] = key;

        _base_config["inputs"].clear();
        _base_config["outputs"].clear();
        
        for (auto& in : inputs)
        {
            _base_config["inputs"].push_back(in->dump());
        }

        for (auto& out : outputs)
        {
            _base_config["outputs"].push_back(out->dump());
        }

        return _base_config;
    }

    void ModuleBase::disconnect() {
        for (size_t i = 0; i < inputs.size(); i++)
        {
            inputs[i]->disconnect();
        }
        for (size_t i = 0; i < outputs.size(); i++)
        {
            outputs[i]->disconnect();
        }
    }

    json ModuleBase::getIdentifier() {
        json jID;
        jID["name"] = _name;
        jID["appendix"] = getAppendix();
        jID["type"] = type;
        jID["prefix"] = std::vector<std::string>(); 
        return jID;
    }
}