#include "Output.h"
namespace mdd {
    int Output::addConnection(std::shared_ptr<IInput> input)
    {
        _connections.push_back(input);
        return _connections.size();
    }
    int Output::removeConnection(std::shared_ptr<IInput> input)
    {
        for (auto it = _connections.begin(); it != _connections.end(); ++it) {
            if ((*it)->getIdentifier() == input->getIdentifier())
            {
                _connections.erase(it);
                return _connections.size();
            }
        }
        return _connections.size();
    }
    Output::Output(IModule* parent, const std::string& name, int appendix, const std::vector<double>& initial) :
            _state(state::UNCHANGED),
            _parent(parent)
    {
        _appendix = appendix;
        _name = name;
        _value = initial;
        _optimizable = false;

        type = "output";
        key = "Output";
    }

    const std::vector<double>& Output::getValue() { return _value; }

    std::vector<double>& Output::setValue() {
        _state = state::CHANGED;
        return _value;
    }
    state Output::setValue(const std::vector<double>& val) {
        if (val != _value)
        {
            _value = val;
            _state = state::CHANGED;
        }
        else {
            _state = state::UNCHANGED;
        }
        return _state;
    }
    bool Output::isOptimizable() {
        return _optimizable;
    }

    void Output::setOptimizability(bool state) {
        _optimizable = state;
    }

    state Output::getState() {
        return _state;
    }
    
    void Output::resetState() {
        _state = state::UNCHANGED;
    }

    std::string Output::getType() {
        return  type;
    }

    std::string Output::getGeneratorKey() {
        return  key;
    }

    std::string Output::setName(const std::string& name) {
        _name = name;
        return getID();
    }

    std::string Output::getName() {
        return  _name;
    }

    std::string Output::setAppendix(int appendix) {
        _appendix = appendix;
        return  getID();
    }

    int Output::getAppendix() {
        return _appendix;
    }

    std::string Output::getID() {
        return _name + std::to_string(_appendix);
    }

    std::string Output::getParentID() {
        return _parent->getID();
    }

    bool Output::connect(std::shared_ptr<IInput> input)
    {
        if (input->getConnection() != nullptr) {
            if (input->getConnection()->getIdentifier() != getIdentifier())
            {
                removeConnection(input);
                input->removeConnection();
            }
        }
        if (input->getConnection() == nullptr)
        {
            addConnection(input);
            input->addConnection(std::make_shared<Output>((*this)));
        }
        return true;
    }
    std::vector<std::shared_ptr<IInput>> Output::getConnections()
    {
        return _connections;
    }
    void Output::disconnect() {
        for (size_t i = 0; i < _connections.size(); i++)
        {
            if (_connections[i] != nullptr)
            {
                _connections[i]->removeConnection(std::make_shared<Output>((*this)));
            }
        }
        _connections.clear();
    }

    bool Output::configure(const std::string& config)
    {
        json jconfig = json::parse(config);
        bool success = false;
        auto jit = jconfig.find("name");
        if (jit != jconfig.end())
        {
            _name = jit.value().get<std::string>();
            success = true;
        }

        jit = jconfig.find("appendix");
        if (jit != jconfig.end())
        {
            _appendix = jit.value().get < int >();
            success = true;
        }

        jit = jconfig.find("optimizable");
        if (jit != jconfig.end())
        {
            _optimizable = jit.value().get < bool >();
            success = true;
        }

        jit = jconfig.find("value");
        if (jit != jconfig.end())
        {
            _value = jit.value().get <std::vector<double> >();
            success = true;
        }
        return success;
    }

    const std::string& Output::getConfiguration()
    {
        _base_config["optimizable"] = _optimizable;
        _base_config["value"] = _value;
        _base_config["type"] = "output";
        _base_config["key"] = "Output";
        return _base_config.dump();
    }

    void Output::load(const json& j)
    {
        configure(j.dump());
        return;
    }

    json Output::dump()
    {
        json jdump = _base_config;
        jdump["ID"] = getIdentifier();
        return jdump;
    }

    json Output::getIdentifier() {
        json jID;
        jID["name"] = _name;
        jID["appendix"] = _appendix;
        jID["type"] = "output";
        jID["prefix"].push_back(_parent->getID());
        return jID;
    }
}